package com.akadia.ejbintro.agent;

import java.io.*;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.ejb.FinderException;
import javax.rmi.PortableRemoteObject;
import javax.sql.DataSource;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.akadia.ejbintro.dept.Dept;
import com.akadia.ejbintro.dept.DeptHome;
import com.akadia.ejbintro.dept.DeptPK;
import com.akadia.ejbintro.emp.Emp;
import com.akadia.ejbintro.emp.EmpHome;
import com.akadia.ejbintro.emp.EmpPK;

/*****************************************************************************
 * <PRE>
 * Akadia EJB introduction
 *
 * The home interface of CompanyAgent
 *
 * (c) Akadia AG 2002
 *
 * </PRE>
 * @author      $Author$
 * @version     $Id$
 * <PRE>
 * $Log$
 * </PRE>
 ****************************************************************************/

public class CompanyAgentBean implements javax.ejb.SessionBean
{
  private static final String DEPT_HOME_NAME="java:comp/env/ejb/TheDeptHomeAlias";
  private static final String EMP_HOME_NAME ="java:comp/env/ejb/TheEmpHomeAlias";
  private static final String DATABASE_NAME ="java:comp/env/jdbc/ejbIntro";

  private DataSource mDataSource = null;
  private DeptHome mDeptHome = null;
  private EmpHome mEmpHome = null;
  private SessionContext mCtx = null;

  public CompanyAgentBean()
  {
  }

  public void ejbCreate()
  {
    // Not implemented.
  }

  public void ejbActivate()
  {
    // Not implemented.
  }

  public void ejbPassivate()
  {
    // Not implemented.
  }

  public void ejbRemove()
  {
    // Not implemented.
  }

  public void setSessionContext( SessionContext ctx )
  {
    mCtx = ctx;
  }

  public void unsetSessionContext()
  {
    mCtx = null;
  }

  public String empToXml( int empNo )
  {
    String xml = null;
    try
    {
      EmpPK pk = new EmpPK( empNo );
      Emp emp = getEmpHome().findByPrimaryKey( pk );
      xml = getXml( emp );
    }
    catch (FinderException e)
    {
      throw new EJBException( "FinderException getting XML: " 
                            + e.getMessage() );
    }
    catch (RemoteException e)
    {
      throw new EJBException( "RemoteException getting XML: " 
                            + e.getMessage() );
    }
    return xml;
  }

  public int getNofEmps( int deptNo )
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    StringBuffer sb = new StringBuffer( 128 );
    int result = -1;
    int i = 1;

    try
    {
      con = getConnection();
      ps = con.prepareStatement( sb
        .append( "SELECT COUNT(*) FROM emp WHERE" )
        .append( " deptNo = ? " )
        .toString()
      );
      ps.setInt( i++, deptNo );
      rs = ps.executeQuery();
      if ( rs.next() )
      {
        result = rs.getInt( 1 );
      }
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( rs != null ) rs.close();
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
    return result;
  }

  public void saveEmp(
      int empNo
    , String eName
    , Date hireDate
    , int deptNo
  )
  {
    Emp emp = null;
    try
    {
      try
      {
        emp = getEmpHome().findByPrimaryKey(new EmpPK( empNo ) );
      }
      catch ( javax.ejb.FinderException ex )
      {
        emp = null;
      }
      if ( emp != null )
      {
        if ( eName != null) emp.setEName( eName );
        if ( hireDate != null) emp.setHireDate( hireDate );
        if ( deptNo != -1 ) emp.setDeptNo( deptNo );
      }
      else
      {
        emp = getEmpHome().create(
            empNo
          , eName
          , hireDate
          , deptNo
        );
      }
    }
    catch ( java.rmi.RemoteException ex )
    {
      throw new EJBException( "RemoteException saving emp: " 
                            + ex.getMessage() );
    }
    catch ( javax.ejb.CreateException ex )
    {
      throw new EJBException( "CreateException saving emp: " 
                            + ex.getMessage() );
    }
  }

  // Set transaction attribute to REQUIRED to handle method
  // within ONE single transaction
  public void deleteDept( int deptNo )
  {
    try
    {
      // Delete employees of given departement
      Collection col = getEmpHome().findByDeptNo( deptNo );
      Iterator itr = col.iterator();
      while (itr.hasNext())
      {
        Object obj = itr.next();
        Emp emp = (Emp) PortableRemoteObject.narrow( obj, Emp.class );
        emp.remove();
      }

      // Delete given departement
      DeptPK pk = new DeptPK( deptNo );
      Dept dept = getDeptHome().findByPrimaryKey( pk );
      dept.remove();
    }
    catch ( FinderException e )
    {
      throw new EJBException( "FinderException deleting dept: " 
                            + e.getMessage() );
    }
    catch ( RemoteException e )
    {
      throw new EJBException( "RemoteException deleting dept: " 
                            + e.getMessage() );
    }
    catch ( RemoveException e )
    {
      throw new EJBException( "RemoveException deleting dept: " 
                            + e.getMessage() );
    }
  }

  private DeptHome getDeptHome()
  {
    try
    {
      if ( mDeptHome == null )
      {
        Context jndiContext = new InitialContext();
        Object obj = jndiContext.lookup( DEPT_HOME_NAME );
        mDeptHome = (DeptHome) PortableRemoteObject.narrow( obj, DeptHome.class );
      }
    }
    catch ( NamingException e )
    {
      throw new EJBException( "NamingException finding dept home: "  
                            + e.getMessage() );
    }
    return mDeptHome;
  }

  private EmpHome getEmpHome()
  {
    try
    {
      if ( mEmpHome == null )
      {
        Context jndiContext = new InitialContext();
        Object obj = jndiContext.lookup( EMP_HOME_NAME );
        mEmpHome = (EmpHome) PortableRemoteObject.narrow( obj, EmpHome.class );
      }
    }
    catch ( NamingException e )
    {
      throw new EJBException( "NamingException finding emp home: "  
                            + e.getMessage() );
    }
    return mEmpHome;
  }

  private Connection getConnection() throws SQLException
  {
    Connection con = null;
    try
    {
      if ( mDataSource == null ) {
        Context jndiCtx = new InitialContext();
        mDataSource = (DataSource) jndiCtx.lookup( DATABASE_NAME );
      }
      con = mDataSource.getConnection();
    }
    catch ( NamingException namEx )
    {
      namEx.printStackTrace();
      throw new EJBException( namEx.getMessage() );
    }
    return con;
  }

  private String getXml( Emp emp )
    throws RemoteException
  {
    StringBuffer sb = new StringBuffer( 512 );
    return sb
      .append( "<EMP>" )
      .append( "<EMP_NO>" )
      .append( emp.getEmpNo() )
      .append( "</EMP_NO>" )
      .append( "<EMP_NAME>" )
      .append( emp.getEName() )
      .append( "</EMP_NAME>" )
      .append( "<HIRE_DATE>" )
      .append( emp.getHireDate() )
      .append( "</HIRE_DATE>" )
      .append( "<DEPT_NO>" )
      .append( emp.getDeptNo() )
      .append( "</DEPT_NO>" )
      .append( "</EMP>" )
      .toString();
  }
}
