package com.akadia.ejbintro.dept;

import java.lang.StringBuffer;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/*****************************************************************************
 * <PRE>
 * Akadia EJB introduction
 *
 * The bean class of Dept
 *
 * (c) Akadia AG 2002
 *
 * </PRE>
 * @author      $Author$
 * @version     $Id$
 * <PRE>
 * $Log$
 * </PRE>
 ****************************************************************************/

public class DeptBean implements javax.ejb.EntityBean
{
  private static final String DATABASE_NAME = "java:comp/env/jdbc/ejbIntro";
  private static final String LOG_CLASS_NAME = "DeptBean: ";

  public EntityContext mCtx = null;
  public int mDeptNo;
  public String mDName;
  public String mLoc;

  public DeptBean()
  {
  }

  public DeptPK ejbCreate(
      int deptNo
    , String dName
    , String loc
  )
    throws CreateException
  {
    log("ejbCreate");
    if ( deptNo < 0)
      throw new CreateException("Invalid parameter");
    mDeptNo = deptNo;
    mDName = dName;
    mLoc = loc;

    Connection con = null;
    PreparedStatement ps = null;
    StringBuffer sb = new StringBuffer( 128 );
    try
    {
      con = this.getConnection();
      ps = con.prepareStatement( sb
        .append( "INSERT INTO dept (" )
        .append( " deptno" )
        .append( ",dname" )
        .append( ",loc" )
        .append( ") VALUES (?,?,?)" )
        .toString()
      );
      ps.setInt( 1, deptNo );
      ps.setString( 2, dName );
      ps.setString( 3, loc );
      if ( ps.executeUpdate() != 1 )
      {
        throw new CreateException( "ejbCreate: Failed to add dept to database");
      }
      DeptPK primaryKey = new DeptPK();
      primaryKey.setDeptNo( deptNo );
      return primaryKey;
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
  }
  
  public void ejbPostCreate(
      int deptNo
    , String dName
    , String loc
  )
  {
    // Do nothing. Required!
    log("ejbPostCreate");
  }

  public DeptPK ejbFindByPrimaryKey(DeptPK primaryKey)
    throws FinderException
  {
    Connection con = null;
    PreparedStatement ps = null;
    StringBuffer sb = new StringBuffer( 128 );
    ResultSet rs = null;
    int i = 1;
    log("ejbFindByPrimaryKey");
    try
    {
      con = this.getConnection();
      ps = con.prepareStatement( sb
        .append( "SELECT " )
        .append( " deptno" )
        .append( " FROM dept" )
        .append( " WHERE deptno = ?" )
        .toString()
      );
      ps.setInt( i++, primaryKey.getDeptNo() );
      rs = ps.executeQuery();
      if ( !rs.next() )
      {
        throw new ObjectNotFoundException( "ejbFindByPrimaryKey: Cannot find dept"
          + " with deptno " + primaryKey.getDeptNo() );
      }
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( rs != null ) rs.close();
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
    return primaryKey;
  }

  public void ejbActivate()
  {
    // Not implemented.
    log("ejbActivate");
  }

  public void ejbPassivate()
  {
    // Not implemented.
    log("ejbPassivate");
  }

  public void ejbLoad()
  {
    DeptPK pk = (DeptPK) mCtx.getPrimaryKey();
    Connection con = null;
    PreparedStatement ps = null;
    StringBuffer sb = new StringBuffer( 128 );
    ResultSet rs = null;
    log("ejbLoad");
    try
    {
      con = this.getConnection();
      ps = con.prepareStatement( sb
        .append( "SELECT " )
        .append( " deptno" )
        .append( ",dname" )
        .append( ",loc" )
        .append( " FROM dept" )
        .append( " WHERE deptno = ?" )
        .toString()
      );
      ps.setInt( 1, pk.getDeptNo() );
      rs = ps.executeQuery();
      if ( rs.next() )
      {
        mDeptNo = rs.getInt( "deptno" );
        mDName = rs.getString( "dname" );
        mLoc = rs.getString( "loc" );
      }
      else
      {
        throw new EJBException( "ejbLoad: Row for deptno " + pk.getDeptNo()
          + " not found in database." );
      }
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( rs != null ) rs.close();
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
  }

  public void ejbStore()
  {
    Connection con = null;
    PreparedStatement ps = null;
    StringBuffer sb = new StringBuffer( 128 );
    log("ejbStore");
    try
    {
      con = this.getConnection();
      ps = con.prepareStatement( sb
        .append( "UPDATE dept SET" )
        .append( " dname = ?" )
        .append( ",loc = ?" )
        .append( " WHERE deptno = ?" )
        .toString()
      );
      ps.setString( 1, mDName );
      ps.setString( 2, mLoc );
      ps.setInt( 3, mDeptNo );
      if ( ps.executeUpdate() != 1 )
      {
        throw new EJBException( "ejbStore: Failed to update dept on database");
      }
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
  }

  public void ejbRemove()
  {
    Connection con = null;
    PreparedStatement ps = null;
    StringBuffer sb = new StringBuffer( 128 );
    log("ejbRemove");
    try
    {
      con = this.getConnection();
      ps = con.prepareStatement( sb
        .append( "DELETE FROM dept WHERE" )
        .append( " deptno = ?" )
        .toString()
      );
      ps.setInt( 1, mDeptNo );
      if ( ps.executeUpdate() != 1 )
      {
        throw new EJBException( "ejbRemove: Failed to delete dept from database");
      }
    }
    catch ( SQLException sqlEx )
    {
      sqlEx.printStackTrace();
      throw new EJBException( sqlEx.getMessage() );
    }
    finally
    {
      try
      {
        if ( ps != null ) ps.close();
        if ( con != null ) con.close();
      }
      catch ( SQLException sqlEx )
      {
        sqlEx.printStackTrace();
      }
    }
  }

  public void setEntityContext(EntityContext ctx)
  {
    log("setEntityContext");
    mCtx = ctx;
  }

  public void unsetEntityContext()
  {
    log("unsetEntityContext");
    mCtx = null;
  }

  public int getDeptNo()
  {
    log("getDeptNo");
    return mDeptNo;
  }

  public void setDeptNo( int deptNo )
  {
    log("setDeptNo");
    mDeptNo = deptNo;
  }

  public String getDName()
  {
    log("getDName");
    return mDName;
  }

  public void setDName( String dName )
  {
    log("setDName");
    mDName = dName;
  }

  public String getLoc()
  {
    log("getLoc");
    return mLoc;
  }

  public void setLoc( String loc )
  {
    log("setLoc");
    mLoc = loc;
  }

  private Connection getConnection()
    throws SQLException
  {
      DataSource ds = null;
      Connection con = null;
      try
      {
        Context jndiCtx = new InitialContext();
        ds = (DataSource) jndiCtx.lookup( DATABASE_NAME );
        con = ds.getConnection();
      }
      catch ( NamingException namEx )
      {
        namEx.printStackTrace();
        throw new EJBException( namEx.getMessage() );
      }
      return con;
  }

  private void log(String msg)
  {
    System.out.println( LOG_CLASS_NAME + msg );
  }

}